@@ -1,3 +1,4 @@
+import json
 import os
 from fabric.api import *
 from fabric.colors import *
@@ -39,37 +40,38 @@ def web_server(*servers):
 
 
 @task
-def plant(VERBOSE=0, test=None):
+def plant(VERBOSE=0):
     """plant beanstalk-stack for project in current work directory
     """
     print magenta('Beanstalk-Stack app plant tool')
     print '=-' * 40
     # Get values
-    setting_values = list()
-    setting_values.append(
-        ('PROJECT_SOURCE_ROOT',
-         prompt('Project Source Root?', default=os.getcwd()),
-         os.getcwd(),
-         'str')
-    )
-    setting_values.append(
-        ('PROJECT_NAME',
-         prompt('Project Name?', default=os.path.split(os.getcwd())[-1]),
-         os.path.split(os.getcwd())[-1],
-         'str')
-    )
-    setting_values.append(
-        ('WEB_SERVERS',
-         prompt('Where to deploy? (Python list/tuple with server address)', validate=validate_web_server),
-         None,
-         'obj')
-    )
-    setting_values.append(
-        ('GIT_IGNORE_PATH',
-         prompt('Where\'s git ignore file?', default='.gitignore', validate=validate_file_existence),
-         '.gitignore',
-         'str')
-    )
+    setting_values = [
+        (
+            'PROJECT_SOURCE_ROOT',
+            prompt('Project Source Root?', default=os.getcwd()),
+            os.getcwd(),
+            'str',
+        ),
+        (
+            'PROJECT_NAME',
+            prompt('Project Name?', default=os.path.split(os.getcwd())[-1]),
+            os.path.split(os.getcwd())[-1],
+            'str',
+        ),
+        (
+            'GIT_IGNORE_PATH',
+            prompt('Where\'s git ignore file?', default='.gitignore', validate=validate_file_existence),
+            '.gitignore',
+            'str',
+        ),
+        (
+            'WEB_SERVERS',
+            prompt('Where to deploy? (Python list/tuple with server address)', validate=validate_web_server),
+            None,
+            'obj',
+        ),
+    ]
     print ''
 
     set_verbose_level(VERBOSE)
@@ -91,14 +93,6 @@ def plant(VERBOSE=0, test=None):
 
     print green('Planted. Grow up now!')
 
-    if test == 't':
-        print '-' * 80
-        with hide('running'):
-            local('ls -l %s' % beanstalk_local_base_path)
-            print '=' * 40
-            local('cat %s' % app_settings_path)
-            local('rm -rf %s' % beanstalk_local_base_path)
-
 
 @task
 @beanstalk_role('app')
@@ -205,18 +199,21 @@ def deploy_remote_core():
     beanstalk_settings = env.beanstalk_settings
     project_name = beanstalk_settings['PROJECT_NAME']
 
+    # Create this app to server
+    run('bsjack server.create_app:%s,VERBOSE=%d' % (project_name, beanstalk_settings['VERBOSE']))
+
     # Call pre-install script
     run_hooked_actions('PRE_INSTALL_ACTIONS')
 
-    # Create this app to server
-    run('bsjack server.create_app:%s' % project_name)
-
+    # Get app info first
+    app_info = json.loads(run('bsjack server.app_info:%s,VERBOSE=%d,USE_JSON=True' %
+                              (project_name, beanstalk_settings['VERBOSE'])))
+    print app_info
     # Send code
     pass
 
-    # Update this app in server
-    run('bsjack server.update_app:%s' % project_name)
-
     # Call post-install script
     run_hooked_actions('POST_INSTALL_ACTIONS')
 
+    # Update this app in server
+    run('bsjack server.update_app:%s,VERBOSE=%d' % (project_name, beanstalk_settings['VERBOSE']))
