@@ -1,8 +1,9 @@
+import json
 import os
 from fabric.api import *
 from fabric.colors import *
 from fabric.contrib.console import confirm
-from soil.fabric import eval_kwargs
+from soil.fabric import eval_kwargs, virtual_env
 from soil.filesystem import temporary_directory
 from beanstalk import IDENTIFIER
 from beanstalk.decorators import beanstalk_role
@@ -10,6 +11,36 @@ from beanstalk.tasks.utils import load_role_settings as load_beanstalk_settings,
 from beanstalk.titles import tool_title, section_title
 
 
+def project_base_path(project_name):
+    beanstalk_settings = env.get('beanstalk_settings', None)
+    if beanstalk_settings is None:
+        abort('Load beanstalk settings first')
+
+    beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
+    project_base = os.path.abspath(os.path.join(beanstalk_stack_base, project_name))
+
+    return project_base
+
+
+def project_source_path(project_name):
+    project_base = project_base_path(project_name)
+    return os.path.join(project_base, 'source/')
+
+
+def project_logs_path(project_name):
+    project_base = project_base_path(project_name)
+    return os.path.join(project_base, 'logs/')
+
+
+def project_venv_path(project_name):
+    project_base = project_base_path(project_name)
+    return os.path.join(project_base, 'venv/')
+
+
+# Tasks
+#-----------------------------------------------------------------------------------------------------------------------
+
+
 @task
 @beanstalk_role('server')
 def setup(settings_path=None, **settings_patches):
@@ -109,60 +140,160 @@ def setup(settings_path=None, **settings_patches):
 @task
 @beanstalk_role('server')
 def create_app(project_name, settings_path=None, **settings_patches):
+    """Create an app
+    """
     settings_patches = eval_kwargs(settings_patches)
-    beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
-    beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
+    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     set_verbose_level()
 
     # Folder paths
-    remote_project_base = os.path.abspath(os.path.join(beanstalk_stack_base, project_name))
-    remote_project_source = os.path.join(remote_project_base, 'source/')
-    remote_project_logs = os.path.join(remote_project_base, 'logs/')
-    #remote_project_venv = os.path.join(remote_project_base, 'venv/')
+    project_base = project_base_path(project_name)
+    project_source = project_source_path(project_name)
+    project_logs = project_logs_path(project_name)
+    project_venv = project_venv_path(project_name)
 
-    if os.path.exists(remote_project_base):
+    if os.path.exists(project_base):
         return
 
     # Create folders
     print section_title('Create folders')
-    local('mkdir -p %s' % remote_project_base)
-    local('mkdir -p %s' % remote_project_source)
-    local('mkdir -p %s' % remote_project_logs)
+    local('mkdir -p %s' % project_base)
+    local('mkdir -p %s' % project_source)
+    local('mkdir -p %s' % project_logs)
 
     # Initialize venv
-    print section_title('Initialize virtual env')
-    with lcd(remote_project_base):
-        local('virtualenv venv')
+    if not os.path.exists(project_venv):
+        print section_title('Initialize virtual env')
+        with lcd(project_base):
+            local('virtualenv venv')
 
     # Initialize git
-    print section_title('Make git repository')
-    with lcd(remote_project_source):
-        local('git init')
-        local('touch .gitignore')
-        local('git add .gitignore')
-        local('git commit -m "Init commit"')
+    if not os.path.exists(os.path.join(project_source, '.git')):
+        print section_title('Make git repository')
+        with lcd(project_source):
+            local('git init')
+            local('touch .gitignore')
+            local('git add .gitignore')
+            local('git commit -m "Init commit"')
 
 
 @task
-@beanstalk_role('derver')
+@beanstalk_role('server')
 def delete_app(project_name, settings_path=None, **settings_patches):
+    """Delete an app
+    """
+    settings_patches = eval_kwargs(settings_patches)
+    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    set_verbose_level()
+
+    # Folder paths
+    project_base = project_base_path(project_name)
+
+    remove = confirm(red('Remove project: %s ?' % project_name), default=False)
+
+    # delete
+    if remove:
+        print section_title('Remove app @ %s' % project_base)
+        local('rm -rf %s' % project_base)
+
+
+@task
+@beanstalk_role('server')
+def update_app(project_name, settings_path=None, **settings_patches):
+    """Update an app (call when u updated the code)
+    """
+    settings_patches = eval_kwargs(settings_patches)
+    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    set_verbose_level()
+
+    print 'update_app: %s' % project_name
+    # TODO: update app
+
+
+@task
+@beanstalk_role('server')
+def reload_app(project_name, settings_path=None, **settings_patches):
+    """Update an app (call when u updated the code)
+    """
+    settings_patches = eval_kwargs(settings_patches)
+    load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    set_verbose_level()
+
+    print section_title('Reload app: %s' % project_name)
+
+    project_base = project_base_path(project_name)
+    print project_base
+    # TODO: touch the uwsgi.ini
+
+
+# disable_app
+
+
+# enable_app
+
+
+@task
+@beanstalk_role('server')
+def build_venv(project_name, settings_path=None, **settings_patches):
+    """Build virtual env of target app.
+    """
     settings_patches = eval_kwargs(settings_patches)
     beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
     beanstalk_stack_base = beanstalk_settings['BEANSTALK_STACK_BASE']
     set_verbose_level()
 
-    # Folder paths
-    remote_project_base = os.path.abspath(os.path.join(beanstalk_stack_base, project_name))
+    # path
+    project_venv = project_venv_path(project_name)
+    requirments_path = os.path.join(project_source_path(project_name), 'requirments.txt')
 
-    remove = confirm('Remove project: %s ?' % project_name, default=False)
+    print section_title('Build virtualenv for %s' % project_name)
 
-    # delete
-    if remove:
-        print 'Remove app @ %s' % remote_project_base
-        local('rm -rf %s' % remote_project_base)
+    # Clean it
+    local('rm -rf %s' % project_venv)
+
+    venv_base, venv_name = os.path.split(project_venv)
+    with lcd(venv_base):
+        local('virtualenv %s' % venv_name)
+        with settings(virtual_env(project_venv)):
+            local('pip intall -r %s' % requirments_path)
 
 
 @task
 @beanstalk_role('server')
-def update_app(project_name):
-    print 'update_app: %s' % project_name
+def app_info(project_name, settings_path=None, **settings_patches):
+    """Get info of an app
+    """
+    settings_patches = eval_kwargs(settings_patches)
+    beanstalk_settings = load_beanstalk_settings(local_settings_path=settings_path, **settings_patches)
+    set_verbose_level()
+
+    # Gather information
+    project_base = project_base_path(project_name)
+    project_source = project_source_path(project_name)
+    project_logs = project_logs_path(project_name)
+    project_venv = project_venv_path(project_name)
+
+    if beanstalk_settings.get('USE_JSON', False):
+        result = {
+            'path': {
+                'base': project_base,
+                'source': project_source,
+                'logs': project_logs,
+                'venv': project_venv,
+            },
+            'uwsgi': {
+                #'port':  #TODO: uwsgi port
+            }
+        }
+        json_string = json.dumps(result)
+
+        if beanstalk_settings.get('PRINT_JSON', True):
+            print json_string
+        return json_string
+    else:
+        print section_title('Beanstalk-stack app: %s' % project_name)
+        print '-=' * 40
+        print 'Base   : %s' % project_base
+        print 'Source : %s' % project_base
+        print 'Logs   : %s' % project_base
+        print 'venv   : %s' % project_base
